@@ -39,7 +39,7 @@
 #include <gssapi.h>
 #include <krb5_err.h>
 
-RCSID("$Id: gssapi.c 16800 2006-03-23 16:53:26Z lha $");
+RCSID("$Id: gssapi.c 21513 2007-07-12 12:45:25Z lha $");
 
 int ftp_do_gss_bindings = 0;
 int ftp_do_gss_delegate = 1;
@@ -48,6 +48,7 @@ struct gss_data {
     gss_ctx_id_t context_hdl;
     char *client_name;
     gss_cred_id_t delegated_cred_handle;
+    void *mech_data;
 };
 
 static int
@@ -277,6 +278,7 @@ gss_adat(void *app_data, void *buf, size_t len)
 }
 
 int gss_userok(void*, char*);
+int gss_session(void*, char*);
 
 struct sec_server_mech gss_server_mech = {
     "GSSAPI",
@@ -292,7 +294,8 @@ struct sec_server_mech gss_server_mech = {
     gss_adat,
     NULL, /* pbsz */
     NULL, /* ccc */
-    gss_userok
+    gss_userok,
+    gss_session
 };
 
 #else /* FTP_SERVER */
@@ -304,12 +307,14 @@ import_name(const char *kname, const char *host, gss_name_t *target_name)
 {
     OM_uint32 maj_stat, min_stat;
     gss_buffer_desc name;
+    char *str;
 
-    name.length = asprintf((char**)&name.value, "%s@%s", kname, host);
-    if (name.length == -1) {
+    name.length = asprintf(&str, "%s@%s", kname, host);
+    if (str == NULL) {
 	printf("Out of memory\n");
 	return AUTH_ERROR;
     }
+    name.value = str;
 
     maj_stat = gss_import_name(&min_stat,
 			       &name,
