@@ -32,7 +32,7 @@
  */
 
 #include "common.h"
-RCSID("$Id: gssmask.c 19078 2006-11-20 18:12:41Z lha $");
+RCSID("$Id: gssmask.c 22887 2008-04-07 18:51:38Z lha $");
 
 /*
  *
@@ -284,14 +284,14 @@ static int
 HandleOP(GoodBye)
 {
     struct handle *h = c->handles;
-    int i = 0;
+    unsigned int i = 0;
 
     while (h) {
 	h = h->next;
 	i++;
     }
 
-    if (i != 0)
+    if (i)
 	logmessage(c, __FILE__, __LINE__, 0,
 		   "Did not toast all resources: %d", i);
     return 1;
@@ -851,7 +851,19 @@ HandleOP(CallExtension)
 static int
 HandleOP(AcquirePKInitCreds)
 {
-    errx(1, "AcquirePKInitCreds");
+    int32_t flags;
+    krb5_data pfxdata;
+
+    ret32(c, flags);
+    retdata(c, pfxdata);
+
+    /* get credentials */
+
+    krb5_data_free(&pfxdata);
+
+    put32(c, -1); /* hResource */
+    put32(c, GSMERR_NOT_SUPPORTED);
+    return 0;
 }
 
 /*
