@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: kx.h 17450 2006-05-05 11:11:43Z lha $ */
+/* $Id: kx.h 20452 2007-04-19 20:04:19Z lha $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -256,12 +256,8 @@ __attribute__ ((format (printf, 3, 4)))
 #endif
 ;
 
-#ifndef KRB4
+int
+kx_get_int(void *f, uint32_t *to, int size, int lsb);
 
 int
-krb_get_int(void *f, uint32_t *to, int size, int lsb);
-
-int
-krb_put_int(uint32_t from, void *to, size_t rem, int size);
-
-#endif
+kx_put_int(uint32_t from, void *to, size_t rem, int size);
