@@ -5,7 +5,7 @@
  */
 
 #include <popper.h>
-RCSID("$Id: pop_parse.c,v 1.8 1996/12/14 20:54:32 assar Exp $");
+RCSID("$Id: pop_parse.c 5530 1999-03-13 21:17:27Z assar $");
 
 /* 
  *  parse:  Parse a raw input line from a POP client 
@@ -22,7 +22,7 @@ pop_parse(POP *p, char *buf)
     for (mp = buf, i = 0; ; i++) {
     
         /*  Skip leading spaces and tabs in the message */
-        while (isspace(*mp))mp++;
+        while (isspace((unsigned char)*mp))mp++;
 
         /*  Are we at the end of the message? */
         if (*mp == 0) break;
@@ -37,7 +37,7 @@ pop_parse(POP *p, char *buf)
         p->pop_parm[i] = mp;
 
         /*  Search for the first space character (end of the token) */
-        while (!isspace(*mp) && *mp) mp++;
+        while (!isspace((unsigned char)*mp) && *mp) mp++;
 
         /*  Delimit the token with a null */
         if (*mp) *mp++ = 0;
