@@ -32,7 +32,7 @@
  */
 
 #include "push_locl.h"
-RCSID("$Id: push.c 14850 2005-04-19 18:00:17Z lha $");
+RCSID("$Id: push.c 22888 2008-04-07 18:51:49Z lha $");
 
 #ifdef KRB4
 static int use_v4 = -1;
@@ -227,7 +227,7 @@ doit(int s,
     size_t from_line_length;
     time_t now;
     struct write_state write_state;
-    int numheaders = 1;
+    unsigned int numheaders = 1;
     char **headers = NULL;
     int i;
     char *tmp = NULL;
