@@ -31,7 +31,7 @@
  * SUCH DAMAGE. 
  */
 
-/* $Id: rsh_locl.h 19033 2006-11-14 09:47:07Z lha $ */
+/* $Id: rsh_locl.h 22945 2008-04-10 09:47:16Z lha $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -106,10 +106,9 @@
 struct krb5_pk_identity;
 struct krb5_pk_cert;
 struct ContentInfo;
+struct AlgorithmIdentifier;
 struct _krb5_krb_auth_data;
 struct krb5_dh_moduli;
-struct krb5_plugin;
-enum plugin_type;
 #include "crypto-headers.h"
 #include <krb5-private.h> /* for _krb5_{get,put}_int */
 #endif
