@@ -53,7 +53,7 @@
 
 #include <config.h>
 
-RCSID("$Id: auth.c 10809 2002-01-18 12:58:49Z joda $");
+RCSID("$Id: auth.c 22895 2008-04-07 18:52:20Z lha $");
 
 #if	defined(AUTHENTICATION)
 #include <stdio.h>
@@ -627,7 +627,8 @@ auth_debug(int mode)
 }
 
 void
-auth_printsub(unsigned char *data, int cnt, unsigned char *buf, int buflen)
+auth_printsub(unsigned char *data, size_t cnt,
+	      unsigned char *buf, size_t buflen)
 {
     Authenticator *ap;
 
@@ -638,7 +639,8 @@ auth_printsub(unsigned char *data, int cnt, unsigned char *buf, int buflen)
 }
 
 void
-auth_gen_printsub(unsigned char *data, int cnt, unsigned char *buf, int buflen)
+auth_gen_printsub(unsigned char *data, size_t cnt,
+		  unsigned char *buf, size_t buflen)
 {
     unsigned char *cp;
     unsigned char tbuf[16];
