@@ -54,7 +54,7 @@
 
 #include <config.h>
 
-RCSID("$Id: encrypt.c 16802 2006-03-23 19:36:31Z lha $");
+RCSID("$Id: encrypt.c 22895 2008-04-07 18:52:20Z lha $");
 
 #if	defined(ENCRYPTION)
 
@@ -965,8 +965,8 @@ encrypt_debug(int mode)
     encrypt_debug_mode = mode;
 }
 
-void encrypt_gen_printsub(unsigned char *data, int cnt, 
-			  unsigned char *buf, int buflen)
+void encrypt_gen_printsub(unsigned char *data, size_t cnt, 
+			  unsigned char *buf, size_t buflen)
 {
     char tbuf[16], *cp;
 
@@ -986,7 +986,8 @@ void encrypt_gen_printsub(unsigned char *data, int cnt,
 }
 
 void
-encrypt_printsub(unsigned char *data, int cnt, unsigned char *buf, int buflen)
+encrypt_printsub(unsigned char *data, size_t cnt, 
+		 unsigned char *buf, size_t buflen)
 {
     Encryptions *ep;
     int type = data[1];
