@@ -55,7 +55,7 @@
 #include <config.h>
 #endif
 
-RCSID("$Id: kerberos.c 17450 2006-05-05 11:11:43Z lha $");
+RCSID("$Id: kerberos.c 22895 2008-04-07 18:52:20Z lha $");
 
 #ifdef	KRB4
 #ifdef HAVE_SYS_TYPES_H
@@ -537,11 +537,12 @@ kerberos4_status(Authenticator *ap, char *name, size_t name_sz, int level)
 #define	ADDC(buf, len, c)	if ((len) > 0) {*(buf)++ = (c); --(len);}
 
 void
-kerberos4_printsub(unsigned char *data, int cnt, unsigned char *buf, int buflen)
+kerberos4_printsub(unsigned char *data, size_t cnt, 
+		   unsigned char *buf, size_t buflen)
 {
     int i;
 
-    buf[buflen-1] = '\0';		/* make sure its NULL terminated */
+    buf[buflen-1] = '\0';		/* make sure it's NULL terminated */
     buflen -= 1;
 
     switch(data[3]) {
