@@ -33,7 +33,7 @@
 
 #include <config.h>
 
-RCSID("$Id: krb4encpwd.c 9681 2001-02-15 04:20:54Z assar $");
+RCSID("$Id: krb4encpwd.c 22895 2008-04-07 18:52:20Z lha $");
 
 #ifdef	KRB4_ENCPWD
 /*
@@ -348,13 +348,12 @@ krb4encpwd_status(ap, name, name_sz, level)
 #define	ADDC(buf, len, c)	if ((len) > 0) {*(buf)++ = (c); --(len);}
 
 	void
-krb4encpwd_printsub(data, cnt, buf, buflen)
-	unsigned char *data, *buf;
-	int cnt, buflen;
+krb4encpwd_printsub(unsigned char *data, size_t cnt, 
+		    unsigned char *buf, size_t buflen)
 {
 	int i;
 
-	buf[buflen-1] = '\0';		/* make sure its NULL terminated */
+	buf[buflen-1] = '\0';		/* make sure it's NULL terminated */
 	buflen -= 1;
 
 	switch(data[3]) {
