@@ -1,5 +1,5 @@
 # Make prototypes from .c files
-# $Id$
+# $Id: make-proto.pl 23023 2008-04-17 10:01:46Z lha $
 
 ##use Getopt::Std;
 require 'getopts.pl';
@@ -11,9 +11,8 @@ my $line = "";
 my $debug = 0;
 my $oproto = 1;
 my $private_func_re = "^_";
-my %depfunction = ();
 
-Getopts('x:m:o:p:dqE:R:P:') || die "foo";
+do Getopts('x:m:o:p:dqE:R:P:') || die "foo";
 
 if($opt_d) {
     $debug = 1;
@@ -26,7 +25,7 @@ if($opt_q) {
 if($opt_R) {
     $private_func_re = $opt_R;
 }
-my %flags = (
+%flags = (
 	  'multiline-proto' => 1,
 	  'header' => 1,
 	  'function-blocking' => 0,
@@ -103,19 +102,11 @@ while(<>) {
 	s/\s+/ /g;
 	if($_ =~ /\)$/){
 	    if(!/^static/ && !/^PRIVATE/){
-		$attr = "";
-		if(m/(.*)(__attribute__\s?\(.*\))/) {
-		    $attr .= " $2";
+		if(/(.*)(__attribute__\s?\(.*\))/) {
+		    $attr = $2;
 		    $_ = $1;
-		}
-		if(m/(.*)\s(\w+DEPRECATED_FUNCTION)\s?(\(.*\))(.*)/) {
-		    $depfunction{$2} = 1;
-		    $attr .= " $2$3";
-		    $_ = "$1 $4";
-		}
-		if(m/(.*)\s(\w+DEPRECATED)(.*)/) {
-		    $attr .= " $2";
-		    $_ = "$1 $3";
+		} else {
+		    $attr = "";
 		}
 		# remove outer ()
 		s/\s*\(/</;
@@ -186,7 +177,6 @@ sub foo {
     local ($arg) = @_;
     $_ = $arg;
     s/.*\/([^\/]*)/$1/;
-    s/.*\\([^\\]*)/$1/;
     s/[^a-zA-Z0-9]/_/g;
     "__" . $_ . "__";
 }
@@ -260,14 +250,8 @@ $private_h_trailer = "";
 
 foreach(sort keys %funcs){
     if(/^(main)$/) { next }
-    if ($funcs{$_} =~ /\^/) {
-	$beginblock = "#ifdef __BLOCKS__\n";
-	$endblock = "#endif /* __BLOCKS__ */\n";
-    } else {
-	$beginblock = $endblock = "";
-    }
     if(!defined($exported{$_}) && /$private_func_re/) {
-	$private_h .= $beginblock . $funcs{$_} . "\n" . $endblock . "\n";
+	$private_h .= $funcs{$_} . "\n\n";
 	if($funcs{$_} =~ /__attribute__/) {
 	    $private_attribute_seen = 1;
 	}
@@ -280,7 +264,7 @@ foreach(sort keys %funcs){
 		$public_h .= "#ifndef HAVE_$fupper\n";
 	    }
 	}
-	$public_h .= $beginblock . $funcs{$_} . "\n" . $endblock;
+	$public_h .= $funcs{$_} . "\n";
 	if($funcs{$_} =~ /__attribute__/) {
 	    $public_attribute_seen = 1;
 	}
@@ -308,82 +292,45 @@ if($flags{"gnuc-attribute"}) {
 ";
     }
 }
-
-my $depstr = "";
-my $undepstr = "";
-foreach (keys %depfunction) {
-    $depstr .= "#ifndef $_
-#if defined(__GNUC__) && ((__GNUC__ > 3) || ((__GNUC__ == 3) && (__GNUC_MINOR__ >= 1 )))
-#define $_(X) __attribute__((__deprecated__))
-#else
-#define $_(X)
-#endif
-#endif
-
-
-";
-    $public_h_trailer .= "#undef $_
-
-";
-    $private_h_trailer .= "#undef $_
-#define $_(X)
-
-";
-}
-
-$public_h_header .= $depstr;
-$private_h_header .= $depstr;
-
-
 if($flags{"cxx"}) {
     $public_h_header .= "#ifdef __cplusplus
 extern \"C\" {
 #endif
 
 ";
-    $public_h_trailer = "#ifdef __cplusplus
+    $public_h_trailer .= "#ifdef __cplusplus
 }
 #endif
 
-" . $public_h_trailer;
+";
 
 }
 if ($opt_E) {
     $public_h_header .= "#ifndef $opt_E
-#ifndef ${opt_E}_FUNCTION
 #if defined(_WIN32)
-#define ${opt_E}_FUNCTION __declspec(dllimport)
-#define ${opt_E}_CALL __stdcall
+#define ${opt_E}_FUNCTION _stdcall __declspec(dllimport)
 #define ${opt_E}_VARIABLE __declspec(dllimport)
 #else
 #define ${opt_E}_FUNCTION
-#define ${opt_E}_CALL
 #define ${opt_E}_VARIABLE
 #endif
 #endif
-#endif
+
 ";
     
     $private_h_header .= "#ifndef $opt_E
-#ifndef ${opt_E}_FUNCTION
 #if defined(_WIN32)
-#define ${opt_E}_FUNCTION __declspec(dllimport)
-#define ${opt_E}_CALL __stdcall
+#define ${opt_E}_FUNCTION _stdcall __declspec(dllimport)
 #define ${opt_E}_VARIABLE __declspec(dllimport)
 #else
 #define ${opt_E}_FUNCTION
-#define ${opt_E}_CALL
 #define ${opt_E}_VARIABLE
 #endif
 #endif
-#endif
 
 ";
 }
     
-$public_h_trailer .= $undepstr;
-$private_h_trailer .= $undepstr;
-
 if ($public_h ne "" && $flags{"header"}) {
     $public_h = $public_h_header . $public_h . 
 	$public_h_trailer . "#endif /* $block */\n";
