@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2004 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1999-2006 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -34,7 +34,7 @@
 #include "kadmin_locl.h"
 #include "kadmin-commands.h"
 
-RCSID("$Id: del_enctype.c,v 1.12 2004/10/31 14:13:17 lha Exp $");
+RCSID("$Id: del_enctype.c 16658 2006-01-25 12:29:46Z lha $");
 
 /*
  * del_enctype principal enctypes...
@@ -119,5 +119,5 @@ out:
     kadm5_free_principal_ent(kadm_handle, &princ);
 out2:
     free (etypes);
-    return 0;
+    return ret != 0;
 }
