@@ -35,7 +35,7 @@
 #include "kadmin-commands.h"
 #include <sl.h>
 
-RCSID("$Id: kadmin.c 17949 2006-08-28 14:20:18Z lha $");
+RCSID("$Id: kadmin.c 22253 2007-12-09 06:00:00Z lha $");
 
 static char *config_file;
 static char *keyfile;
@@ -166,8 +166,11 @@ main(int argc, char **argv)
     argc -= optidx;
     argv += optidx;
 
-    if (config_file == NULL)
-	config_file = HDB_DB_DIR "/kdc.conf";
+    if (config_file == NULL) {
+	asprintf(&config_file, "%s/kdc.conf", hdb_db_dir(context));
+	if (config_file == NULL)
+	    errx(1, "out of memory");
+    }
 
     ret = krb5_prepend_config_files_default(config_file, &files);
     if (ret)
