@@ -33,7 +33,7 @@
 
 #include "kadmin_locl.h"
 
-RCSID("$Id: kadmind.c 15587 2005-07-07 22:06:50Z lha $");
+RCSID("$Id: kadmind.c 22250 2007-12-09 05:57:31Z lha $");
 
 static char *check_library  = NULL;
 static char *check_function = NULL;
@@ -116,9 +116,12 @@ main(int argc, char **argv)
     argc -= optidx;
     argv += optidx;
 
-    if (config_file == NULL)
-	config_file = HDB_DB_DIR "/kdc.conf";
-
+    if (config_file == NULL) {
+	asprintf(&config_file, "%s/kdc.conf", hdb_db_dir(context));
+	if (config_file == NULL)
+	    errx(1, "out of memory");
+    }
+    
     ret = krb5_prepend_config_files_default(config_file, &files);
     if (ret)
 	krb5_err(context, 1, ret, "getting configuration files");
