@@ -33,7 +33,7 @@
 
 #include "kdc_locl.h"
 
-RCSID("$Id: connect.c 20958 2007-06-07 04:35:05Z lha $");
+RCSID("$Id: connect.c 22877 2008-04-07 18:50:56Z lha $");
 
 /* Should we enable the HTTP hack? */
 int enable_http = -1;
@@ -695,7 +695,7 @@ handle_http_tcp (krb5_context context,
 	    "<TITLE>404 Not found</TITLE>\r\n"
 	    "<H1>404 Not found</H1>\r\n"
 	    "That page doesn't exist, maybe you are looking for "
-	    "<A HREF=\"http://www.pdc.kth.se/heimdal/\">Heimdal</A>?\r\n";
+	    "<A HREF=\"http://www.h5l.org/\">Heimdal</A>?\r\n";
 	kdc_log(context, config, 0, "HTTP request from %s is non KDC request", d->addr_string);
 	kdc_log(context, config, 5, "HTTP request: %s", t);
 	free(data);
@@ -823,7 +823,7 @@ loop(krb5_context context,
      krb5_kdc_configuration *config)
 {
     struct descr *d;
-    int ndescr;
+    unsigned int ndescr;
 
     ndescr = init_sockets(context, config, &d);
     if(ndescr <= 0)
