@@ -33,7 +33,7 @@
 
 #include "kdc_locl.h"
 
-RCSID("$Id: kaserver.c 17904 2006-08-23 11:45:16Z lha $");
+RCSID("$Id: kaserver.c 23110 2008-04-27 18:51:17Z lha $");
 
 #include <krb5-v4compat.h>
 #include <rx.h>
@@ -191,19 +191,28 @@ init_reply_header (struct rx_header *hdr,
     reply_hdr->serviceid = hdr->serviceid;
 }
 
+/*
+ * Create an error `reply using for the packet `hdr' with the error
+ * `error code.
+ */
 static void
 make_error_reply (struct rx_header *hdr,
-		  uint32_t ret,
+		  uint32_t error,
 		  krb5_data *reply)
 
 {
-    krb5_storage *sp;
     struct rx_header reply_hdr;
+    krb5_error_code ret;
+    krb5_storage *sp;
 
     init_reply_header (hdr, &reply_hdr, HT_ABORT, HF_LAST);
     sp = krb5_storage_emem();
+    if (sp == NULL)
+	return;
     ret = encode_rx_header (&reply_hdr, sp);
-    krb5_store_int32(sp, ret);
+    if (ret)
+	return;
+    krb5_store_int32(sp, error);
     krb5_storage_to_data (sp, reply);
     krb5_storage_free (sp);
 }
@@ -357,7 +366,7 @@ create_reply_ticket (krb5_context context,
 	DES_cblock deskey;
 	
 	memcpy (&deskey, key->keyvalue.data, sizeof(deskey));
-	DES_set_key (&deskey, &schedule);
+	DES_set_key_unchecked (&deskey, &schedule);
 	DES_pcbc_encrypt (enc_data.data,
 			  enc_data.data,
 			  enc_data.length,
@@ -515,7 +524,7 @@ do_authenticate (krb5_context context,
 	
 	/* try to decode the `request' */
 	memcpy (&key, ckey->key.keyvalue.data, sizeof(key));
-	DES_set_key (&key, &schedule);
+	DES_set_key_unchecked (&key, &schedule);
 	DES_pcbc_encrypt (request.data,
 			  request.data,
 			  request.length,
@@ -792,7 +801,7 @@ do_getticket (krb5_context context,
 
     /* decrypt the times */
     memcpy(&session, ad.session.keyvalue.data, sizeof(session));
-    DES_set_key (&session, &schedule);
+    DES_set_key_unchecked (&session, &schedule);
     DES_ecb_encrypt (times.data,
 		     times.data,
 		     &schedule,
