@@ -41,7 +41,7 @@
 
 #include "check-common.h"
 
-RCSID("$Id: check-der.c 20438 2007-04-19 15:18:07Z lha $");
+RCSID("$Id: check-der.c 21359 2007-06-27 08:15:41Z lha $");
 
 static int
 cmp_integer (void *a, void *b)
@@ -65,12 +65,11 @@ test_integer (void)
 	{NULL, 1, "\xff"},
 	{NULL, 2, "\xff\x01"},
 	{NULL, 2, "\x00\xff"},
-	{NULL, 4, "\x80\x00\x00\x00"},
 	{NULL, 4, "\x7f\xff\xff\xff"}
     };
 
     int values[] = {0, 127, 128, 256, -128, -129, -1, -255, 255,
-		    0x80000000, 0x7fffffff};
+		    0x7fffffff};
     int i, ret;
     int ntests = sizeof(tests) / sizeof(*tests);
 
@@ -90,7 +89,6 @@ test_integer (void)
 
     for (i = 0; i < ntests; ++i)
 	free (tests[i].name);
-
     return ret;
 }
 
@@ -194,7 +192,7 @@ test_unsigned (void)
 
     unsigned int values[] = {0, 127, 128, 256, 512, 32768, 
 			     0x80000000, 0x7fffffff};
-    int i;
+    int i, ret;
     int ntests = sizeof(tests) / sizeof(*tests);
 
     for (i = 0; i < ntests; ++i) {
@@ -204,12 +202,15 @@ test_unsigned (void)
 	    errx(1, "malloc");
     }
 
-    return generic_test (tests, ntests, sizeof(int),
-			 (generic_encode)der_put_unsigned,
-			 (generic_length)der_length_unsigned,
-			 (generic_decode)der_get_unsigned,
-			 (generic_free)NULL,
-			 cmp_unsigned);
+    ret = generic_test (tests, ntests, sizeof(int),
+			(generic_encode)der_put_unsigned,
+			(generic_length)der_length_unsigned,
+			(generic_decode)der_get_unsigned,
+			(generic_free)NULL,
+			cmp_unsigned);
+    for (i = 0; i < ntests; ++i) 
+	free (tests[i].name);
+    return ret;
 }
 
 static int
@@ -233,18 +234,21 @@ test_octet_string (void)
 	{NULL, 8, "\x01\x23\x45\x67\x89\xab\xcd\xef"}
     };
     int ntests = sizeof(tests) / sizeof(*tests);
+    int ret;
 
     tests[0].val = &s1;
     asprintf (&tests[0].name, "a octet string");
     if (tests[0].name == NULL)
 	errx(1, "malloc");
 
-    return generic_test (tests, ntests, sizeof(heim_octet_string),
-			 (generic_encode)der_put_octet_string,
-			 (generic_length)der_length_octet_string,
-			 (generic_decode)der_get_octet_string,
-			 (generic_free)der_free_octet_string,
-			 cmp_octet_string);
+    ret = generic_test (tests, ntests, sizeof(heim_octet_string),
+			(generic_encode)der_put_octet_string,
+			(generic_length)der_length_octet_string,
+			(generic_decode)der_get_octet_string,
+			(generic_free)der_free_octet_string,
+			cmp_octet_string);
+    free(tests[0].name);
+    return ret;
 }
 
 static int
@@ -270,6 +274,7 @@ test_bmp_string (void)
 	{NULL, 4, "\x00\x20\x00\x20"}
     };
     int ntests = sizeof(tests) / sizeof(*tests);
+    int ret;
 
     tests[0].val = &s1;
     asprintf (&tests[0].name, "a bmp string");
@@ -280,12 +285,60 @@ test_bmp_string (void)
     if (tests[1].name == NULL)
 	errx(1, "malloc");
 
-    return generic_test (tests, ntests, sizeof(heim_bmp_string),
-			 (generic_encode)der_put_bmp_string,
-			 (generic_length)der_length_bmp_string,
-			 (generic_decode)der_get_bmp_string,
-			 (generic_free)der_free_bmp_string,
-			 cmp_bmp_string);
+    ret = generic_test (tests, ntests, sizeof(heim_bmp_string),
+			(generic_encode)der_put_bmp_string,
+			(generic_length)der_length_bmp_string,
+			(generic_decode)der_get_bmp_string,
+			(generic_free)der_free_bmp_string,
+			cmp_bmp_string);
+    free(tests[0].name);
+    free(tests[1].name);
+    return ret;
+}
+
+static int
+cmp_universal_string (void *a, void *b)
+{
+    heim_universal_string *oa = (heim_universal_string *)a;
+    heim_universal_string *ob = (heim_universal_string *)b;
+
+    return der_heim_universal_string_cmp(oa, ob);
+}
+
+static uint32_t universal_d1[] = { 32 };
+static uint32_t universal_d2[] = { 32, 32 };
+
+static int
+test_universal_string (void)
+{
+    heim_universal_string s1 = { 1, universal_d1 };
+    heim_universal_string s2 = { 2, universal_d2 };
+
+    struct test_case tests[] = {
+	{NULL, 4, "\x00\x00\x00\x20"},
+	{NULL, 8, "\x00\x00\x00\x20\x00\x00\x00\x20"}
+    };
+    int ntests = sizeof(tests) / sizeof(*tests);
+    int ret;
+
+    tests[0].val = &s1;
+    asprintf (&tests[0].name, "a universal string");
+    if (tests[0].name == NULL)
+	errx(1, "malloc");
+    tests[1].val = &s2;
+    asprintf (&tests[1].name, "second universal string");
+    if (tests[1].name == NULL)
+	errx(1, "malloc");
+
+    ret = generic_test (tests, ntests, sizeof(heim_universal_string),
+			(generic_encode)der_put_universal_string,
+			(generic_length)der_length_universal_string,
+			(generic_decode)der_get_universal_string,
+			(generic_free)der_free_universal_string,
+			cmp_universal_string);
+    free(tests[0].name);
+    free(tests[1].name);
+    return ret;
 }
 
 static int
@@ -305,19 +358,21 @@ test_general_string (void)
     struct test_case tests[] = {
 	{NULL, 11, "\x54\x65\x73\x74\x20\x55\x73\x65\x72\x20\x31"}
     };
-    int ntests = sizeof(tests) / sizeof(*tests);
+    int ret, ntests = sizeof(tests) / sizeof(*tests);
 
     tests[0].val = &s1;
     asprintf (&tests[0].name, "the string \"%s\"", s1);
     if (tests[0].name == NULL)
 	errx(1, "malloc");
 
-    return generic_test (tests, ntests, sizeof(unsigned char *),
-			 (generic_encode)der_put_general_string,
-			 (generic_length)der_length_general_string,
-			 (generic_decode)der_get_general_string,
-			 (generic_free)der_free_general_string,
-			 cmp_general_string);
+    ret = generic_test (tests, ntests, sizeof(unsigned char *),
+			(generic_encode)der_put_general_string,
+			(generic_length)der_length_general_string,
+			(generic_decode)der_get_general_string,
+			(generic_free)der_free_general_string,
+			cmp_general_string);
+    free(tests[0].name);
+    return ret;
 }
 
 static int
@@ -337,7 +392,7 @@ test_generalized_time (void)
 	{NULL, 15, "19851106210627Z"}
     };
     time_t values[] = {0, 500159187};
-    int i;
+    int i, ret;
     int ntests = sizeof(tests) / sizeof(*tests);
 
     for (i = 0; i < ntests; ++i) {
@@ -347,12 +402,15 @@ test_generalized_time (void)
 	    errx(1, "malloc");
     }
 
-    return generic_test (tests, ntests, sizeof(time_t),
-			 (generic_encode)der_put_generalized_time,
-			 (generic_length)der_length_generalized_time,
-			 (generic_decode)der_get_generalized_time,
-			 (generic_free)NULL,
-			 cmp_generalized_time);
+    ret = generic_test (tests, ntests, sizeof(time_t),
+			(generic_encode)der_put_generalized_time,
+			(generic_length)der_length_generalized_time,
+			(generic_decode)der_get_generalized_time,
+			(generic_free)NULL,
+			cmp_generalized_time);
+    for (i = 0; i < ntests; ++i)
+	free(tests[i].name);
+    return ret;
 }
 
 static int
@@ -381,7 +439,7 @@ test_oid (void)
 	{ 3, oid_comp3 },
 	{ 2, oid_comp4 }
     };
-    int i;
+    int i, ret;
     int ntests = sizeof(tests) / sizeof(*tests);
 
     for (i = 0; i < ntests; ++i) {
@@ -391,12 +449,15 @@ test_oid (void)
 	    errx(1, "malloc");
     }
 
-    return generic_test (tests, ntests, sizeof(heim_oid),
-			 (generic_encode)der_put_oid,
-			 (generic_length)der_length_oid,
-			 (generic_decode)der_get_oid,
-			 (generic_free)der_free_oid,
-			 test_cmp_oid);
+    ret = generic_test (tests, ntests, sizeof(heim_oid),
+			(generic_encode)der_put_oid,
+			(generic_length)der_length_oid,
+			(generic_decode)der_get_oid,
+			(generic_free)der_free_oid,
+			test_cmp_oid);
+    for (i = 0; i < ntests; ++i)
+	free(tests[i].name);
+    return ret;
 }
 
 static int
@@ -414,7 +475,7 @@ test_bit_string (void)
     heim_bit_string values[] = {
 	{ 0, "" }
     };
-    int i;
+    int i, ret;
     int ntests = sizeof(tests) / sizeof(*tests);
 
     for (i = 0; i < ntests; ++i) {
@@ -424,12 +485,15 @@ test_bit_string (void)
 	    errx(1, "malloc");
     }
 
-    return generic_test (tests, ntests, sizeof(heim_bit_string),
-			 (generic_encode)der_put_bit_string,
-			 (generic_length)der_length_bit_string,
-			 (generic_decode)der_get_bit_string,
-			 (generic_free)der_free_bit_string,
-			 test_cmp_bit_string);
+    ret = generic_test (tests, ntests, sizeof(heim_bit_string),
+			(generic_encode)der_put_bit_string,
+			(generic_length)der_length_bit_string,
+			(generic_decode)der_get_bit_string,
+			(generic_free)der_free_bit_string,
+			test_cmp_bit_string);
+    for (i = 0; i < ntests; ++i)
+	free(tests[i].name);
+    return ret;
 }
 
 static int
@@ -460,8 +524,10 @@ test_heim_integer (void)
 	{ 1, "\x01", 0 },
 	{ 1, "\x80", 0 }
     };
-    int i;
+    int i, ret;
     int ntests = sizeof(tests) / sizeof(tests[0]);
+    size_t size;
+    heim_integer i2;
 
     for (i = 0; i < ntests; ++i) {
 	tests[i].val = &values[i];
@@ -470,12 +536,75 @@ test_heim_integer (void)
 	    errx(1, "malloc");
     }
 
-    return generic_test (tests, ntests, sizeof(heim_integer),
-			 (generic_encode)der_put_heim_integer,
-			 (generic_length)der_length_heim_integer,
-			 (generic_decode)der_get_heim_integer,
-			 (generic_free)der_free_heim_integer,
-			 test_cmp_heim_integer);
+    ret = generic_test (tests, ntests, sizeof(heim_integer),
+			(generic_encode)der_put_heim_integer,
+			(generic_length)der_length_heim_integer,
+			(generic_decode)der_get_heim_integer,
+			(generic_free)der_free_heim_integer,
+			test_cmp_heim_integer);
+    for (i = 0; i < ntests; ++i) 
+	free (tests[i].name);
+    if (ret)
+	return ret;
+
+    /* test zero length integer (BER format) */
+    ret = der_get_heim_integer(NULL, 0, &i2, &size);
+    if (ret)
+	errx(1, "der_get_heim_integer");
+    if (i2.length != 0)
+	errx(1, "der_get_heim_integer wrong length");
+    der_free_heim_integer(&i2);
+
+    return 0;
+}
+
+static int
+test_cmp_boolean (void *a, void *b)
+{
+    return !!*(int *)a != !!*(int *)b;
+}
+
+static int
+test_boolean (void)
+{
+    struct test_case tests[] = {
+	{NULL, 1, "\xff"},
+	{NULL, 1, "\x00"}
+    };
+
+    int values[] = { 1, 0 };
+    int i, ret;
+    int ntests = sizeof(tests) / sizeof(tests[0]);
+    size_t size;
+    heim_integer i2;
+
+    for (i = 0; i < ntests; ++i) {
+	tests[i].val = &values[i];
+	asprintf (&tests[i].name, "heim_boolean %d", i);
+	if (tests[i].name == NULL)
+	    errx(1, "malloc");
+    }
+
+    ret = generic_test (tests, ntests, sizeof(int),
+			(generic_encode)der_put_boolean,
+			(generic_length)der_length_boolean,
+			(generic_decode)der_get_boolean,
+			(generic_free)NULL,
+			test_cmp_boolean);
+    for (i = 0; i < ntests; ++i) 
+	free (tests[i].name);
+    if (ret)
+	return ret;
+
+    /* test zero length integer (BER format) */
+    ret = der_get_heim_integer(NULL, 0, &i2, &size);
+    if (ret)
+	errx(1, "der_get_heim_integer");
+    if (i2.length != 0)
+	errx(1, "der_get_heim_integer wrong length");
+    der_free_heim_integer(&i2);
+
+    return 0;
 }
 
 static int
@@ -545,7 +674,8 @@ static int
 check_fail_bmp_string(void)
 {
     struct test_case tests[] = {
-	{NULL, 1, "\x00", "odd length bmpstring"}
+	{NULL, 1, "\x00", "odd (1) length bmpstring"},
+	{NULL, 3, "\x00\x00\x00", "odd (3) length bmpstring"}
     };
     int ntests = sizeof(tests) / sizeof(*tests);
 
@@ -692,6 +822,7 @@ test_heim_int_format(void)
 	"\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF",
 	0
     };
+    heim_integer f;
     int ret = 0;
 
     ret += check_heim_integer_same(p, p, &bni);
@@ -702,6 +833,21 @@ test_heim_int_format(void)
     ret += check_heim_integer_same("01", "01", &i3);
     ret += check_heim_integer_same("1", "01", &i3);
 
+    {
+	int r;
+	r = der_parse_hex_heim_integer("-", &f);
+	if (r == 0) {
+	    der_free_heim_integer(&f);
+	    ret++;
+	}
+	/* used to cause UMR */
+	r = der_parse_hex_heim_integer("00", &f);
+	if (r == 0)
+	    der_free_heim_integer(&f);
+	else
+	    ret++;
+    }
+
     return ret;
 }
 
@@ -791,6 +937,118 @@ check_trailing_nul(void)
     return 0;
 }
 
+static int
+test_misc_cmp(void)
+{
+    int ret;
+
+    /* diffrent lengths are diffrent */
+    {
+	const heim_octet_string os1 = { 1, "a" } , os2 = { 0, NULL };
+	ret = der_heim_octet_string_cmp(&os1, &os2);
+	if (ret == 0)
+	    return 1;
+    }
+    /* diffrent data are diffrent */
+    {
+	const heim_octet_string os1 = { 1, "a" } , os2 = { 1, "b" };
+	ret = der_heim_octet_string_cmp(&os1, &os2);
+	if (ret == 0)
+	    return 1;
+    }
+    /* diffrent lengths are diffrent */
+    {
+	const heim_bit_string bs1 = { 8, "a" } , bs2 = { 7, "a" };
+	ret = der_heim_bit_string_cmp(&bs1, &bs2);
+	if (ret == 0)
+	    return 1;
+    }
+    /* diffrent data are diffrent */
+    {
+	const heim_bit_string bs1 = { 7, "\x0f" } , bs2 = { 7, "\x02" };
+	ret = der_heim_bit_string_cmp(&bs1, &bs2);
+	if (ret == 0)
+	    return 1;
+    }
+    /* diffrent lengths are diffrent */
+    {
+	uint16_t data = 1;
+	heim_bmp_string bs1 = { 1, NULL } , bs2 = { 0, NULL };
+	bs1.data = &data;
+	ret = der_heim_bmp_string_cmp(&bs1, &bs2);
+	if (ret == 0)
+	    return 1;
+    }
+    /* diffrent lengths are diffrent */
+    {
+	uint32_t data;
+	heim_universal_string us1 = { 1, NULL } , us2 = { 0, NULL };
+	us1.data = &data;
+	ret = der_heim_universal_string_cmp(&us1, &us2);
+	if (ret == 0)
+	    return 1;
+    }
+    /* same */
+    {
+	uint32_t data = (uint32_t)'a';
+	heim_universal_string us1 = { 1, NULL } , us2 = { 1, NULL };
+	us1.data = &data;
+	us2.data = &data;
+	ret = der_heim_universal_string_cmp(&us1, &us2);
+	if (ret != 0)
+	    return 1;
+    }
+
+    return 0;
+}
+
+static int
+corner_generalized_time(void)
+{
+    const char *str = "760520140000Z";
+    size_t size;
+    time_t t;
+    int ret;
+
+    ret = der_get_generalized_time((const unsigned char*)str, strlen(str),
+				   &t, &size);
+    if (ret)
+	return 1;
+    return 0;
+}
+
+static int
+corner_tag(void)
+{
+    struct {
+	int ok;
+	const char *ptr;
+	size_t len;
+    } tests[] = { 
+	{ 1, "\x00", 1 },
+	{ 0, "\xff", 1 },
+	{ 0, "\xff\xff\xff\xff\xff\xff\xff\xff", 8 }
+    };
+    int i, ret;
+    Der_class cl;
+    Der_type ty;
+    unsigned int tag;
+    size_t size;
+
+    for (i = 0; i < sizeof(tests)/sizeof(tests[0]); i++) {
+	ret = der_get_tag((const unsigned char*)tests[i].ptr, 
+			  tests[i].len, &cl, &ty, &tag, &size);
+	if (ret) {
+	    if (tests[i].ok)
+		errx(1, "failed while shouldn't");
+	} else {
+	    if (!tests[i].ok)
+		errx(1, "passed while shouldn't");
+	}
+    }
+    return 0;
+}
+
 int
 main(int argc, char **argv)
 {
@@ -801,11 +1059,13 @@ main(int argc, char **argv)
     ret += test_unsigned ();
     ret += test_octet_string ();
     ret += test_bmp_string ();
+    ret += test_universal_string ();
     ret += test_general_string ();
     ret += test_generalized_time ();
     ret += test_oid ();
     ret += test_bit_string();
     ret += test_heim_integer();
+    ret += test_boolean();
 
     ret += check_fail_unsigned();
     ret += check_fail_integer();
@@ -821,6 +1081,9 @@ main(int argc, char **argv)
     ret += test_heim_int_format();
     ret += test_heim_oid_format();
     ret += check_trailing_nul();
+    ret += test_misc_cmp();
+    ret += corner_generalized_time();
+    ret += corner_tag();
 
     return ret;
 }
