@@ -25,7 +25,7 @@
 #include <ctype.h>
 #include <errno.h>
 
-RCSID("$Id: editline.c 15581 2005-07-07 20:55:18Z lha $");
+RCSID("$Id: editline.c 22898 2008-04-07 18:52:32Z lha $");
 
 /*
 **  Manifest constants.
@@ -278,12 +278,10 @@ static void
 columns(int ac, unsigned char **av)
 {
     unsigned char	*p;
-    int		i;
-    int		j;
+    size_t	len;
     int		k;
-    int		len;
     int		skip;
-    int		longest;
+    unsigned int longest, i, j;
     int		cols;
 
     /* Find longest name, determine column count from that. */
