@@ -28,7 +28,7 @@
 
 #include "mech_locl.h"
 #include <heim_threads.h>
-RCSID("$Id: gss_mech_switch.c 21700 2007-07-26 19:08:34Z lha $");
+RCSID("$Id: gss_mech_switch.c 22875 2008-04-07 18:50:48Z lha $");
 
 #ifndef _PATH_GSS_MECH
 #define _PATH_GSS_MECH	"/etc/gss/mech"
@@ -46,7 +46,7 @@ static int
 _gss_string_to_oid(const char* s, gss_OID oid)
 {
 	int			number_count, i, j;
-	int			byte_count;
+	size_t			byte_count;
 	const char		*p, *q;
 	char			*res;
 
@@ -118,7 +118,7 @@ _gss_string_to_oid(const char* s, gss_OID oid)
 				 * The number is encoded in seven bit chunks.
 				 */
 				unsigned int t;
-				int bytes;
+				unsigned int bytes;
 
 				bytes = 0;
 				for (t = number; t; t >>= 7)
