@@ -33,7 +33,7 @@
 
 #include "ntlm/ntlm.h"
 
-RCSID("$Id: accept_sec_context.c 20698 2007-05-30 15:07:09Z lha $");
+RCSID("$Id: accept_sec_context.c 22519 2008-01-24 11:51:29Z lha $");
 
 /*
  *
@@ -43,7 +43,6 @@ OM_uint32
 _gss_ntlm_allocate_ctx(OM_uint32 *minor_status, ntlm_ctx *ctx)
 {
     OM_uint32 maj_stat;
-    /* krb5_error_code ret; */
 
     *ctx = calloc(1, sizeof(**ctx));
 
@@ -112,6 +111,13 @@ _gss_ntlm_accept_sec_context
 	    return major_status;
 	*context_handle = (gss_ctx_id_t)ctx;
 	
+	/* check if the mechs is allowed by remote service */
+	major_status = (*ctx->server->nsi_probe)(minor_status, ctx->ictx, NULL);
+	if (major_status) {
+	    _gss_ntlm_delete_sec_context(minor_status, context_handle, NULL);
+	    return major_status;
+	}
+
 	data.data = input_token_buffer->value;
 	data.length = input_token_buffer->length;
 	
@@ -123,6 +129,7 @@ _gss_ntlm_accept_sec_context
 	}
 
 	if ((type1.flags & NTLM_NEG_UNICODE) == 0) {
+	    heim_ntlm_free_type1(&type1);
 	    _gss_ntlm_delete_sec_context(minor_status, context_handle, NULL);
 	    *minor_status = EINVAL;
 	    return GSS_S_FAILURE;
@@ -181,12 +188,29 @@ _gss_ntlm_accept_sec_context
 					     ctx->ictx,
 					     &type3,
 					     &session);
-	heim_ntlm_free_type3(&type3);
 	if (maj_stat) {
+	    heim_ntlm_free_type3(&type3);
 	    _gss_ntlm_delete_sec_context(minor_status, context_handle, NULL);
 	    return maj_stat;
 	}
 
+	if (src_name) {
+	    ntlm_name n = calloc(1, sizeof(*n));
+	    if (n) {
+		n->user = strdup(type3.username);
+		n->domain = strdup(type3.targetname);
+	    }
+	    if (n == NULL || n->user == NULL || n->domain == NULL) {
+		heim_ntlm_free_type3(&type3);
+		_gss_ntlm_delete_sec_context(minor_status, 
+					     context_handle, NULL);
+		return maj_stat;
+	    }
+	    *src_name = (gss_name_t)n;
+	}	    
+
+	heim_ntlm_free_type3(&type3);
+
 	ret = krb5_data_copy(&ctx->sessionkey, 
 			     session.data, session.length);
 	if (ret) {	
