@@ -33,10 +33,10 @@
 
 #include "ntlm/ntlm.h"
 
-RCSID("$Id: init_sec_context.c 21206 2007-06-20 06:01:48Z lha $");
+RCSID("$Id: init_sec_context.c 22382 2007-12-30 12:13:17Z lha $");
 
 static int
-from_file(const char *fn, const char *domain,
+from_file(const char *fn, const char *target_domain, 
 	  char **username, struct ntlm_buf *key)
 {	  
     char *str, buf[1024];
@@ -53,12 +53,13 @@ from_file(const char *fn, const char *domain,
 	    continue;
 	str = NULL;
 	d = strtok_r(buf, ":", &str);
-	if (d && strcasecmp(domain, d) != 0)
+	if (d && strcasecmp(target_domain, d) != 0)
 	    continue;
 	u = strtok_r(NULL, ":", &str);
 	p = strtok_r(NULL, ":", &str);
 	if (u == NULL || p == NULL)
 	    continue;
+
 	*username = strdup(u);
 
 	heim_ntlm_nt_key(p, key);
@@ -73,15 +74,20 @@ from_file(const char *fn, const char *domain,
 }
 
 static int
-get_user_file(const char *domain, char **username, struct ntlm_buf *key)
+get_user_file(const ntlm_name target_name, 
+	      char **username, struct ntlm_buf *key)
 {
-    const char *fn = NULL;
+    const char *fn;
+
+    if (issuid())
+	return ENOENT;
+
+    fn = getenv("NTLM_USER_FILE");
+    if (fn == NULL)
+	return ENOENT;
+    if (from_file(fn, target_name->domain, username, key) == 0)
+	return 0;
 
-    if (!issuid()) {
-	fn = getenv("NTLM_USER_FILE");
-	if (fn != NULL && from_file(fn, domain, username, key) == 0)
-	    return 0;
-    }
     return ENOENT;
 }
 
@@ -90,7 +96,7 @@ get_user_file(const char *domain, char **username, struct ntlm_buf *key)
  */
 
 static int
-get_user_ccache(const char *domain, char **username, struct ntlm_buf *key)
+get_user_ccache(const ntlm_name name, char **username, struct ntlm_buf *key)
 {
     krb5_principal client;
     krb5_context context = NULL;
@@ -125,7 +131,7 @@ get_user_ccache(const char *domain, char **username, struct ntlm_buf *key)
 
     ret = krb5_make_principal(context, &mcreds.server,
 			      krb5_principal_get_realm(context, client),
-			      "@ntlm-key", domain, NULL);
+			      "@ntlm-key", name->domain, NULL);
     krb5_free_principal(context, client);
     if (ret)
 	goto out;
@@ -165,6 +171,61 @@ out:
     return ret;
 }
 
+int
+_gss_ntlm_get_user_cred(const ntlm_name target_name,
+			ntlm_cred *rcred)
+{
+    ntlm_cred cred;
+    int ret;
+ 
+    cred = calloc(1, sizeof(*cred));
+    if (cred == NULL)
+	return ENOMEM;
+    
+    ret = get_user_file(target_name, &cred->username, &cred->key);
+    if (ret)
+	ret = get_user_ccache(target_name, &cred->username, &cred->key);
+    if (ret) {
+	free(cred);
+	return ret;
+    }
+    
+    cred->domain = strdup(target_name->domain);
+    *rcred = cred;
+
+    return ret;
+}
+
+static int
+_gss_copy_cred(ntlm_cred from, ntlm_cred *to)
+{
+    *to = calloc(1, sizeof(*to));
+    if (*to == NULL)
+	return ENOMEM;
+    (*to)->username = strdup(from->username);
+    if ((*to)->username == NULL) {
+	free(*to);
+	return ENOMEM;
+    }
+    (*to)->domain = strdup(from->domain);
+    if ((*to)->domain == NULL) {
+	free((*to)->username);
+	free(*to);
+	return ENOMEM;
+    }
+    (*to)->key.data = malloc(from->key.length);
+    if ((*to)->key.data == NULL) {
+	free((*to)->domain);
+	free((*to)->username);
+	free(*to);
+	return ENOMEM;
+    }
+    memcpy((*to)->key.data, from->key.data, from->key.length);
+    (*to)->key.length = from->key.length;
+
+    return 0;
+}
+
 OM_uint32
 _gss_ntlm_init_sec_context
            (OM_uint32 * minor_status,
@@ -195,10 +256,10 @@ _gss_ntlm_init_sec_context
 	*actual_mech_type = GSS_C_NO_OID;
 
     if (*context_handle == GSS_C_NO_CONTEXT) {
-	krb5_error_code ret;
 	struct ntlm_type1 type1;
 	struct ntlm_buf data;
 	uint32_t flags = 0;
+	int ret;
 	
 	ctx = calloc(1, sizeof(*ctx));
 	if (ctx == NULL) {
@@ -207,11 +268,12 @@ _gss_ntlm_init_sec_context
 	}
 	*context_handle = (gss_ctx_id_t)ctx;
 
-	ret = get_user_file(name->domain, 
-			    &ctx->client.username, &ctx->client.key);
-	if (ret)
-	    ret = get_user_ccache(name->domain,
-				  &ctx->client.username, &ctx->client.key);
+	if (initiator_cred_handle != GSS_C_NO_CREDENTIAL) {
+	    ntlm_cred cred = (ntlm_cred)initiator_cred_handle;
+	    ret = _gss_copy_cred(cred, &ctx->client);
+	} else
+	    ret = _gss_ntlm_get_user_cred(name, &ctx->client);
+
 	if (ret) {
 	    _gss_ntlm_delete_sec_context(minor_status, context_handle, NULL);
 	    *minor_status = ret;
@@ -274,7 +336,7 @@ _gss_ntlm_init_sec_context
 
 	memset(&type3, 0, sizeof(type3));
 
-	type3.username = ctx->client.username;
+	type3.username = ctx->client->username;
 	type3.flags = type2.flags;
 	type3.targetname = type2.targetname;
 	type3.ws = rk_UNCONST("workstation");
@@ -298,12 +360,12 @@ _gss_ntlm_init_sec_context
 
 		ret = heim_ntlm_calculate_ntlm2_sess(nonce,
 						     type2.challange,
-						     ctx->client.key.data,
+						     ctx->client->key.data,
 						     &type3.lm,
 						     &type3.ntlm);
 	    } else {
-		ret = heim_ntlm_calculate_ntlm1(ctx->client.key.data, 
-						ctx->client.key.length,
+		ret = heim_ntlm_calculate_ntlm1(ctx->client->key.data, 
+						ctx->client->key.length,
 						type2.challange,
 						&type3.ntlm);
 
@@ -314,8 +376,8 @@ _gss_ntlm_init_sec_context
 		return GSS_S_FAILURE;
 	    }
 
-	    ret = heim_ntlm_build_ntlm1_master(ctx->client.key.data, 
-					       ctx->client.key.length,
+	    ret = heim_ntlm_build_ntlm1_master(ctx->client->key.data, 
+					       ctx->client->key.length,
 					       &sessionkey,
 					       &type3.sessionkey);
 	    if (ret) {
@@ -364,9 +426,9 @@ _gss_ntlm_init_sec_context
 		return GSS_S_FAILURE;
 	    }
 
-	    ret = heim_ntlm_calculate_ntlm2(ctx->client.key.data,
-					    ctx->client.key.length,
-					    ctx->client.username,
+	    ret = heim_ntlm_calculate_ntlm2(ctx->client->key.data,
+					    ctx->client->key.length,
+					    ctx->client->username,
 					    name->domain,
 					    type2.challange,
 					    &type2.targetinfo,
