@@ -33,7 +33,7 @@
 
 #include "ntlm/ntlm.h"
 
-RCSID("$Id: inquire_cred.c 21250 2007-06-21 01:27:46Z lha $");
+RCSID("$Id: inquire_cred.c 22148 2007-12-04 17:59:29Z lha $");
 
 OM_uint32 _gss_ntlm_inquire_cred
            (OM_uint32 * minor_status,
@@ -57,6 +57,9 @@ OM_uint32 _gss_ntlm_inquire_cred
     if (mechanisms)
 	*mechanisms = GSS_C_NO_OID_SET;
 
+    if (cred_handle == GSS_C_NO_CREDENTIAL)
+	return GSS_S_NO_CRED;
+
     if (mechanisms) {
         ret = gss_create_empty_oid_set(minor_status, mechanisms);
         if (ret)
