@@ -119,6 +119,11 @@ _gss_ntlm_get_mic (
 	const gss_buffer_t /*message_buffer*/,
 	gss_buffer_t message_token );
 
+int
+_gss_ntlm_get_user_cred (
+	const ntlm_name /*target_name*/,
+	ntlm_cred */*rcred*/);
+
 OM_uint32
 _gss_ntlm_import_name (
 	OM_uint32 * /*minor_status*/,
