@@ -8,6 +8,11 @@
 #define __attribute__(x)
 #endif
 
+int
+_hx509_AlgorithmIdentifier_cmp (
+	const AlgorithmIdentifier */*p*/,
+	const AlgorithmIdentifier */*q*/);
+
 int
 _hx509_Certificate_cmp (
 	const Certificate */*p*/,
@@ -92,6 +97,12 @@ _hx509_cert_set_release (
 	_hx509_cert_release_func /*release*/,
 	void */*ctx*/);
 
+int
+_hx509_cert_to_env (
+	hx509_context /*context*/,
+	hx509_cert /*cert*/,
+	hx509_env */*env*/);
+
 int
 _hx509_certs_keys_add (
 	hx509_context /*context*/,
@@ -176,6 +187,18 @@ _hx509_create_signature_bitstring (
 	AlgorithmIdentifier */*signatureAlgorithm*/,
 	heim_bit_string */*sig*/);
 
+int
+_hx509_expr_eval (
+	hx509_context /*context*/,
+	hx509_env /*env*/,
+	struct hx_expr */*expr*/);
+
+void
+_hx509_expr_free (struct hx_expr */*expr*/);
+
+struct hx_expr *
+_hx509_expr_parse (const char */*buf*/);
+
 int
 _hx509_find_extension_subject_key_id (
 	const Certificate */*issuer*/,
@@ -248,6 +271,12 @@ _hx509_lock_get_passwords (hx509_lock /*lock*/);
 hx509_certs
 _hx509_lock_unlock_certs (hx509_lock /*lock*/);
 
+struct hx_expr *
+_hx509_make_expr (
+	enum hx_expr_op /*op*/,
+	void */*arg1*/,
+	void */*arg2*/);
+
 int
 _hx509_map_file (
 	const char */*fn*/,
@@ -269,12 +298,14 @@ _hx509_match_keys (
 int
 _hx509_name_cmp (
 	const Name */*n1*/,
-	const Name */*n2*/);
+	const Name */*n2*/,
+	int */*c*/);
 
 int
 _hx509_name_ds_cmp (
 	const DirectoryString */*ds1*/,
-	const DirectoryString */*ds2*/);
+	const DirectoryString */*ds2*/,
+	int */*diff*/);
 
 int
 _hx509_name_from_Name (
@@ -352,6 +383,12 @@ _hx509_private_key_exportable (hx509_private_key /*key*/);
 int
 _hx509_private_key_free (hx509_private_key */*key*/);
 
+BIGNUM *
+_hx509_private_key_get_internal (
+	hx509_context /*context*/,
+	hx509_private_key /*key*/,
+	const char */*type*/);
+
 int
 _hx509_private_key_init (
 	hx509_private_key */*key*/,
@@ -470,6 +507,12 @@ _hx509_request_to_pkcs10 (
 	const hx509_private_key /*signer*/,
 	heim_octet_string */*request*/);
 
+hx509_revoke_ctx
+_hx509_revoke_ref (hx509_revoke_ctx /*ctx*/);
+
+void
+_hx509_sel_yyerror (char */*s*/);
+
 int
 _hx509_set_cert_attribute (
 	hx509_context /*context*/,
