@@ -8,11 +8,13 @@
 extern "C" {
 #endif
 
-#ifndef HX509_LIB_FUNCTION
+#ifndef HX509_LIB
 #if defined(_WIN32)
-#define HX509_LIB_FUNCTION _stdcall
+#define HX509_LIB_FUNCTION _stdcall __declspec(dllimport)
+#define HX509_LIB_VARIABLE __declspec(dllimport)
 #else
 #define HX509_LIB_FUNCTION
+#define HX509_LIB_VARIABLE
 #endif
 #endif
 
@@ -183,6 +185,7 @@ hx509_cert_cmp (
 
 int
 hx509_cert_find_subjectAltName_otherName (
+	hx509_context /*context*/,
 	hx509_cert /*cert*/,
 	const heim_oid */*oid*/,
 	hx509_octet_string_list */*list*/);
@@ -192,9 +195,16 @@ hx509_cert_free (hx509_cert /*cert*/);
 
 int
 hx509_cert_get_SPKI (
+	hx509_context /*context*/,
 	hx509_cert /*p*/,
 	SubjectPublicKeyInfo */*spki*/);
 
+int
+hx509_cert_get_SPKI_AlgorithmIdentifier (
+	hx509_context /*context*/,
+	hx509_cert /*p*/,
+	AlgorithmIdentifier */*alg*/);
+
 hx509_cert_attribute
 hx509_cert_get_attribute (
 	hx509_cert /*cert*/,
@@ -230,6 +240,9 @@ hx509_cert_get_subject (
 	hx509_cert /*p*/,
 	hx509_name */*name*/);
 
+int
+hx509_cert_have_private_key (hx509_cert /*p*/);
+
 int
 hx509_cert_init (
 	hx509_context /*context*/,
@@ -305,7 +318,7 @@ int
 hx509_certs_iter (
 	hx509_context /*context*/,
 	hx509_certs /*certs*/,
-	int (*/*fn*/)(hx509_context, void *, hx509_cert),
+	int (*/*func*/)(hx509_context, void *, hx509_cert),
 	void */*ctx*/);
 
 int
@@ -402,7 +415,7 @@ hx509_cms_verify_signed (
 	const void */*data*/,
 	size_t /*length*/,
 	const heim_octet_string */*signedContent*/,
-	hx509_certs /*store*/,
+	hx509_certs /*pool*/,
 	heim_oid */*contentType*/,
 	heim_octet_string */*content*/,
 	hx509_certs */*signer_certs*/);
@@ -553,18 +566,32 @@ hx509_crypto_set_random_key (
 int
 hx509_env_add (
 	hx509_context /*context*/,
-	hx509_env /*env*/,
+	hx509_env */*env*/,
 	const char */*key*/,
 	const char */*value*/);
 
+int
+hx509_env_add_binding (
+	hx509_context /*context*/,
+	hx509_env */*env*/,
+	const char */*key*/,
+	hx509_env /*list*/);
+
+const char *
+hx509_env_find (
+	hx509_context /*context*/,
+	hx509_env /*env*/,
+	const char */*key*/);
+
+hx509_env
+hx509_env_find_binding (
+	hx509_context /*context*/,
+	hx509_env /*env*/,
+	const char */*key*/);
+
 void
 hx509_env_free (hx509_env */*env*/);
 
-int
-hx509_env_init (
-	hx509_context /*context*/,
-	hx509_env */*env*/);
-
 const char *
 hx509_env_lfind (
 	hx509_context /*context*/,
@@ -580,6 +607,9 @@ hx509_err (
 	const char */*fmt*/,
 	...);
 
+void
+hx509_free_error_string (char */*str*/);
+
 void
 hx509_free_octet_string_list (hx509_octet_string_list */*list*/);
 
@@ -651,6 +681,11 @@ hx509_lock_set_prompter (
 	hx509_prompter_fct /*prompt*/,
 	void */*data*/);
 
+int
+hx509_name_binary (
+	const hx509_name /*name*/,
+	heim_octet_string */*os*/);
+
 int
 hx509_name_cmp (
 	hx509_name /*n1*/,
@@ -684,12 +719,6 @@ hx509_name_to_Name (
 	const hx509_name /*from*/,
 	Name */*to*/);
 
-int
-hx509_name_to_der_name (
-	const hx509_name /*name*/,
-	void **/*data*/,
-	size_t */*length*/);
-
 int
 hx509_name_to_string (
 	const hx509_name /*name*/,
@@ -782,13 +811,6 @@ hx509_pem_write (
 	const void */*data*/,
 	size_t /*size*/);
 
-void
-hx509_print_func (
-	hx509_vprint_func /*func*/,
-	void */*ctx*/,
-	const char */*fmt*/,
-	...);
-
 void
 hx509_print_stdout (
 	void */*ctx*/,
@@ -814,6 +836,17 @@ hx509_query_match_cmp_func (
 	int (*/*func*/)(void *, hx509_cert),
 	void */*ctx*/);
 
+int
+hx509_query_match_eku (
+	hx509_query */*q*/,
+	const heim_oid */*eku*/);
+
+int
+hx509_query_match_expr (
+	hx509_context /*context*/,
+	hx509_query */*q*/,
+	const char */*expr*/);
+
 int
 hx509_query_match_friendly_name (
 	hx509_query */*q*/,
@@ -901,6 +934,9 @@ hx509_signature_md5 (void);
 const AlgorithmIdentifier *
 hx509_signature_rsa (void);
 
+const AlgorithmIdentifier *
+hx509_signature_rsa_pkcs1_x509 (void);
+
 const AlgorithmIdentifier *
 hx509_signature_rsa_with_md2 (void);
 
@@ -1030,6 +1066,12 @@ hx509_verify_signature (
 	const heim_octet_string */*data*/,
 	const heim_octet_string */*sig*/);
 
+void
+hx509_xfree (void */*ptr*/);
+
+int
+yywrap (void);
+
 #ifdef __cplusplus
 }
 #endif
