@@ -32,7 +32,7 @@
  */
 
 #include "hx_locl.h"
-RCSID("$Id: hxtool.c 21342 2007-06-26 14:21:34Z lha $");
+RCSID("$Id: hxtool.c 22677 2008-03-13 17:35:49Z lha $");
 
 #include <hxtool-commands.h>
 #include <sl.h>
@@ -141,6 +141,26 @@ peer_strings(hx509_context context,
     free(val);
 }
 
+/*
+ *
+ */
+
+static int
+pem_reader(hx509_context context, const char *type, 
+	   const hx509_pem_header *headers,
+	   const void *data , size_t length, void *ctx)
+{
+    heim_octet_string *c = (heim_octet_string *)ctx;
+
+    c->data = malloc(length);
+    if (c->data == NULL)
+	return ENOMEM;
+    memcpy(c->data, data, length);
+    c->length = length;
+
+    return 0;
+}
+
 /*
  *
  */
@@ -166,17 +186,6 @@ cms_verify_sd(struct cms_verify_sd_options *opt, int argc, char **argv)
     hx509_lock_init(context, &lock);
     lock_strings(lock, &opt->pass_strings);
 
-    ret = _hx509_map_file(argv[0], &p, &sz, NULL);
-    if (ret)
-	err(1, "map_file: %s: %d", argv[0], ret);
-
-    if (opt->signed_content_string) {
-	ret = _hx509_map_file_os(opt->signed_content_string, &signeddata, NULL);
-	if (ret)
-	    err(1, "map_file: %s: %d", opt->signed_content_string, ret);
-	sd = &signeddata;
-    }
-
     ret = hx509_verify_init_ctx(context, &ctx);
 
     ret = hx509_certs_init(context, "MEMORY:cms-anchors", 0, NULL, &anchors);
@@ -185,8 +194,32 @@ cms_verify_sd(struct cms_verify_sd_options *opt, int argc, char **argv)
     certs_strings(context, "anchors", anchors, lock, &opt->anchors_strings);
     certs_strings(context, "store", store, lock, &opt->certificate_strings);
 
-    co.data = p;
-    co.length = sz;
+    if (opt->pem_flag) {
+	FILE *f;
+	
+	f = fopen(argv[0], "r");
+	if (f == NULL)
+	    err(1, "Failed to open file %s", argv[0]);
+
+	ret = hx509_pem_read(context, f, pem_reader, &co);
+	fclose(f);
+	if (ret)
+	    errx(1, "PEM reader failed: %d", ret);
+    } else {
+	ret = _hx509_map_file(argv[0], &p, &sz, NULL);
+	if (ret)
+	    err(1, "map_file: %s: %d", argv[0], ret);
+
+	co.data = p;
+	co.length = sz;
+    }
+
+    if (opt->signed_content_string) {
+	ret = _hx509_map_file_os(opt->signed_content_string, &signeddata, NULL);
+	if (ret)
+	    errx(1, "map_file: %s: %d", opt->signed_content_string, ret);
+	sd = &signeddata;
+    }
 
     if (opt->content_info_flag) {
 	heim_octet_string uwco;
@@ -235,7 +268,10 @@ cms_verify_sd(struct cms_verify_sd_options *opt, int argc, char **argv)
 	errx(1, "hx509_write_file: %d", ret);
 
     der_free_octet_string(&c);
-    _hx509_unmap_file(p, sz);
+    if (opt->pem_flag)
+	der_free_octet_string(&co);
+    else
+	_hx509_unmap_file(p, sz);
     if (sd)
 	_hx509_unmap_file_os(sd);
 
@@ -360,7 +396,8 @@ cms_create_sd(struct cms_create_sd_options *opt, int argc, char **argv)
 	FILE *f;
 
 	hx509_pem_add_header(&header, "Content-disposition", 
-			     opt->detached_signature_flag ? "detached" : "inline");
+			     opt->detached_signature_flag ?
+			     "detached" : "inline");
 	hx509_pem_add_header(&header, "Signer", signer_name);
 
 	f = fopen(argv[1], "w");
@@ -721,7 +758,7 @@ verify_f(hx509_context hxcontext, void *ctx, hx509_cert c)
     if (ret) {
 	char *s = hx509_get_error_string(hxcontext, ret);
 	printf("verify_path: %s: %d\n", s, ret);
-	free(s);
+	hx509_free_error_string(s);
 	v->errors++;
     } else
 	printf("path ok\n");
@@ -885,6 +922,17 @@ query(struct query_options *opt, int argc, char **argv)
     if (opt->friendlyname_string)
 	hx509_query_match_friendly_name(q, opt->friendlyname_string);
 
+    if (opt->eku_string) {
+	heim_oid oid;
+
+	parse_oid(opt->eku_string, NULL, &oid);
+
+	ret = hx509_query_match_eku(q, &oid);
+	if (ret)
+	    errx(1, "hx509_query_match_eku: %d", ret);
+	der_free_oid(&oid);
+    }
+
     if (opt->private_key_flag)
 	hx509_query_match_option(q, HX509_QUERY_OPTION_PRIVATE_KEY);
 
@@ -894,6 +942,9 @@ query(struct query_options *opt, int argc, char **argv)
     if (opt->digitalSignature_flag)
 	hx509_query_match_option(q, HX509_QUERY_OPTION_KU_DIGITALSIGNATURE);
 
+    if (opt->expr_string)
+	hx509_query_match_expr(context, q, opt->expr_string);
+
     ret = hx509_certs_find(context, certs, q, &c);
     hx509_query_free(context, q);
     if (ret)
@@ -1002,7 +1053,7 @@ verify_o(hx509_context hxcontext, void *ctx, hx509_cert c)
     if (ret) {
 	char *s = hx509_get_error_string(hxcontext, ret);
 	printf("ocsp_verify: %s: %d\n", s, ret);
-	free(s);
+	hx509_free_error_string(s);
     } else
 	printf("expire: %d\n", (int)expiration);
 
