@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997 - 2004 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997 - 2007 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -34,7 +34,7 @@
 #include "iprop.h"
 #include <rtbl.h>
 
-RCSID("$Id: ipropd_master.c 21826 2007-08-06 02:21:48Z lha $");
+RCSID("$Id: ipropd_master.c 23074 2008-04-23 15:06:51Z lha $");
 
 static krb5_log_facility *log_facility;
 
@@ -128,16 +128,19 @@ check_acl (krb5_context context, const char *name)
     FILE *fp;
     char buf[256];
     int ret = 1;
+    char *slavefile;
 
+    asprintf(&slavefile, "%s/slaves", hdb_db_dir(context));
 
     fn = krb5_config_get_string_default(context,
 					NULL,
-					KADM5_SLAVE_ACL,
+					slavefile,
 					"kdc",
 					"iprop-acl",
 					NULL);
 
     fp = fopen (fn, "r");
+    free(slavefile);
     if (fp == NULL)
 	return 1;
     while (fgets(buf, sizeof(buf), fp) != NULL) {
@@ -405,6 +408,8 @@ send_are_you_there (krb5_context context, slave *s)
     if (s->flags & (SLAVE_F_DEAD|SLAVE_F_AYT))
 	return 0;
 
+    krb5_warnx(context, "slave %s missing, sending AYT", s->name);
+
     s->flags |= SLAVE_F_AYT;
 
     data.data = buf;
@@ -592,19 +597,26 @@ process_msg (krb5_context context, slave *s, int log_fd,
 static FILE *
 open_stats(krb5_context context)
 {
+    char *statfile = NULL;
     const char *fn;
+    FILE *f;
 
     if (slave_stats_file)
 	fn = slave_stats_file;
-    else
+    else {
+	asprintf(&statfile,  "%s/slaves-stats", hdb_db_dir(context));
 	fn = krb5_config_get_string_default(context,
 					    NULL,
-					    KADM5_SLAVE_STATS,
+					    statfile,
 					    "kdc",
 					    "iprop-stats",
 					    NULL);
+    }
+    f = fopen(fn, "w");
+    if (statfile)
+	free(statfile);
 
-    return fopen(fn, "w");
+    return f;
 }
 
 static void
@@ -748,8 +760,11 @@ main(int argc, char **argv)
 
     setup_signal();
 
-    if (config_file == NULL)
-	config_file = HDB_DB_DIR "/kdc.conf";
+    if (config_file == NULL) {
+	asprintf(&config_file, "%s/kdc.conf", hdb_db_dir(context));
+	if (config_file == NULL)
+	    errx(1, "out of memory");
+    }
 
     ret = krb5_prepend_config_files_default(config_file, &files);
     if (ret)
@@ -896,10 +911,8 @@ main(int argc, char **argv)
 		    slave_dead(context, p);
 	    } else if (slave_gone_p (p))
 		slave_dead(context, p);
-	    else if (slave_missing_p (p)) {
-		krb5_warnx(context, "slave %s missing, sending AYT", p->name);
+	    else if (slave_missing_p (p))
 		send_are_you_there (context, p);
-	    }
 	}
 
 	if (ret && FD_ISSET(listen_fd, &readset)) {
@@ -915,8 +928,8 @@ main(int argc, char **argv)
     else if(exit_flag == SIGINT || exit_flag == SIGTERM)
 	krb5_warnx(context, "%s terminated", getprogname());
     else
-	krb5_warnx(context, "%s unexpected exit reason: %d", 
-		   getprogname(), exit_flag);
+	krb5_warnx(context, "%s unexpected exit reason: %ld", 
+		   getprogname(), (long)exit_flag);
 
     write_master_down(context);
 
