@@ -33,7 +33,7 @@
 
 #include "iprop.h"
 
-RCSID("$Id: ipropd_slave.c 21818 2007-08-03 14:18:04Z lha $");
+RCSID("$Id: ipropd_slave.c 22773 2008-03-24 19:13:46Z lha $");
 
 static krb5_log_facility *log_facility;
 static char *server_time_lost = "5 min";
@@ -234,12 +234,15 @@ receive_loop (krb5_context context,
 
 	ret = kadm5_log_replay (server_context,
 				op, vers, len, sp);
-	if (ret)
-	    krb5_warn (context, ret,
+	if (ret) {
+	    char *s = krb5_get_error_message(server_context->context, ret);
+	    krb5_warnx (context,
 		       "kadm5_log_replay: %ld. Lost entry entry, "
-		       "Database out of sync ?",
+		       "Database out of sync ?: %s (%d)",
+			(long)vers, s ? s : "unknown error", ret);
+	    krb5_xfree(s);
+	}
 
-		       (long)vers);
 	{
 	    /* 
 	     * Make sure the krb5_log_replay does the right thing wrt
@@ -477,8 +480,11 @@ main(int argc, char **argv)
 
     setup_signal();
 
-    if (config_file == NULL)
-	config_file = HDB_DB_DIR "/kdc.conf";
+    if (config_file == NULL) {
+	asprintf(&config_file, "%s/kdc.conf", hdb_db_dir(context));
+	if (config_file == NULL)
+	    errx(1, "out of memory");
+    }
 
     ret = krb5_prepend_config_files_default(config_file, &files);
     if (ret)
@@ -619,8 +625,8 @@ main(int argc, char **argv)
     else if(exit_flag == SIGINT || exit_flag == SIGTERM)
 	krb5_warnx(context, "%s terminated", getprogname());
     else
-	krb5_warnx(context, "%s unexpected exit reason: %d", 
-		   getprogname(), exit_flag);
+	krb5_warnx(context, "%s unexpected exit reason: %ld", 
+		   getprogname(), (long)exit_flag);
 
     return 0;
 }
