@@ -12,11 +12,13 @@
 extern "C" {
 #endif
 
-#ifndef KRB5_LIB_FUNCTION
+#ifndef KRB5_LIB
 #if defined(_WIN32)
-#define KRB5_LIB_FUNCTION _stdcall
+#define KRB5_LIB_FUNCTION _stdcall __declspec(dllimport)
+#define KRB5_LIB_VARIABLE __declspec(dllimport)
 #else
 #define KRB5_LIB_FUNCTION
+#define KRB5_LIB_VARIABLE
 #endif
 #endif
 
@@ -670,6 +672,12 @@ krb5_cc_initialize (
 	krb5_ccache /*id*/,
 	krb5_principal /*primary_principal*/);
 
+krb5_error_code
+krb5_cc_move (
+	krb5_context /*context*/,
+	krb5_ccache /*from*/,
+	krb5_ccache /*to*/);
+
 krb5_error_code KRB5_LIB_FUNCTION
 krb5_cc_new_unique (
 	krb5_context /*context*/,
@@ -743,6 +751,11 @@ krb5_cc_store_cred (
 	krb5_ccache /*id*/,
 	krb5_creds */*creds*/);
 
+krb5_error_code
+krb5_cc_switch (
+	krb5_context /*context*/,
+	krb5_ccache /*id*/);
+
 krb5_error_code KRB5_LIB_FUNCTION
 krb5_change_password (
 	krb5_context /*context*/,
@@ -758,14 +771,14 @@ krb5_check_transited (
 	krb5_const_realm /*client_realm*/,
 	krb5_const_realm /*server_realm*/,
 	krb5_realm */*realms*/,
-	int /*num_realms*/,
+	unsigned int /*num_realms*/,
 	int */*bad_realm*/);
 
 krb5_error_code KRB5_LIB_FUNCTION
 krb5_check_transited_realms (
 	krb5_context /*context*/,
 	const char *const */*realms*/,
-	int /*num_realms*/,
+	unsigned int /*num_realms*/,
 	int */*bad_realm*/);
 
 krb5_error_code KRB5_LIB_FUNCTION
@@ -1329,6 +1342,13 @@ krb5_digest_init_request (
 	krb5_realm /*realm*/,
 	krb5_ccache /*ccache*/);
 
+krb5_error_code
+krb5_digest_probe (
+	krb5_context /*context*/,
+	krb5_realm /*realm*/,
+	krb5_ccache /*ccache*/,
+	unsigned */*flags*/);
+
 krb5_boolean
 krb5_digest_rep_get_status (
 	krb5_context /*context*/,
@@ -1449,14 +1469,14 @@ krb5_domain_x500_decode (
 	krb5_context /*context*/,
 	krb5_data /*tr*/,
 	char ***/*realms*/,
-	int */*num_realms*/,
+	unsigned int */*num_realms*/,
 	const char */*client_realm*/,
 	const char */*server_realm*/);
 
 krb5_error_code KRB5_LIB_FUNCTION
 krb5_domain_x500_encode (
 	char **/*realms*/,
-	int /*num_realms*/,
+	unsigned int /*num_realms*/,
 	krb5_data */*encoding*/);
 
 krb5_error_code KRB5_LIB_FUNCTION
@@ -1606,6 +1626,9 @@ krb5_err (
 	...)
     __attribute__ ((noreturn, format (printf, 4, 5)));
 
+krb5_error_code KRB5_LIB_FUNCTION 
+    __attribute__((deprecated)) krb5_free_creds_contents (krb5_context context, krb5_creds *c);
+
 krb5_error_code KRB5_LIB_FUNCTION
 krb5_error_from_rd_error (
 	krb5_context /*context*/,
@@ -1694,11 +1717,6 @@ krb5_free_creds (
 	krb5_context /*context*/,
 	krb5_creds */*c*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
-krb5_free_creds_contents (
-	krb5_context /*context*/,
-	krb5_creds */*c*/);
-
 void KRB5_LIB_FUNCTION
 krb5_free_data (
 	krb5_context /*context*/,
@@ -2805,45 +2823,45 @@ krb5_openlog (
 krb5_error_code
 krb5_pac_add_buffer (
 	krb5_context /*context*/,
-	struct krb5_pac */*p*/,
+	krb5_pac /*p*/,
 	uint32_t /*type*/,
 	const krb5_data */*data*/);
 
 void
 krb5_pac_free (
 	krb5_context /*context*/,
-	struct krb5_pac */*pac*/);
+	krb5_pac /*pac*/);
 
 krb5_error_code
 krb5_pac_get_buffer (
 	krb5_context /*context*/,
-	struct krb5_pac */*p*/,
+	krb5_pac /*p*/,
 	uint32_t /*type*/,
 	krb5_data */*data*/);
 
 krb5_error_code
 krb5_pac_get_types (
 	krb5_context /*context*/,
-	struct krb5_pac */*p*/,
+	krb5_pac /*p*/,
 	size_t */*len*/,
 	uint32_t **/*types*/);
 
 krb5_error_code
 krb5_pac_init (
 	krb5_context /*context*/,
-	struct krb5_pac **/*pac*/);
+	krb5_pac */*pac*/);
 
 krb5_error_code
 krb5_pac_parse (
 	krb5_context /*context*/,
 	const void */*ptr*/,
 	size_t /*len*/,
-	struct krb5_pac **/*pac*/);
+	krb5_pac */*pac*/);
 
 krb5_error_code
 krb5_pac_verify (
 	krb5_context /*context*/,
-	const struct krb5_pac */*pac*/,
+	const krb5_pac /*pac*/,
 	time_t /*authtime*/,
 	krb5_const_principal /*principal*/,
 	const krb5_keyblock */*server*/,
@@ -2895,6 +2913,13 @@ krb5_password_key_proc (
 	krb5_const_pointer /*keyseed*/,
 	krb5_keyblock **/*key*/);
 
+krb5_error_code
+krb5_plugin_register (
+	krb5_context /*context*/,
+	enum krb5_plugin_type /*type*/,
+	const char */*name*/,
+	void */*symbol*/);
+
 krb5_error_code KRB5_LIB_FUNCTION
 krb5_prepend_config_files (
 	const char */*filelist*/,
@@ -2951,6 +2976,12 @@ krb5_principal_match (
 	krb5_const_principal /*princ*/,
 	krb5_const_principal /*pattern*/);
 
+krb5_error_code KRB5_LIB_FUNCTION
+krb5_principal_set_realm (
+	krb5_context /*context*/,
+	krb5_principal /*principal*/,
+	krb5_const_realm /*realm*/);
+
 void KRB5_LIB_FUNCTION
 krb5_principal_set_type (
 	krb5_context /*context*/,
