@@ -32,7 +32,7 @@
  */
 
 #include "krb5_locl.h"
-RCSID("$Id: plugin.c 21702 2007-07-26 19:13:53Z lha $");
+RCSID("$Id: plugin.c 22033 2007-11-10 10:39:47Z lha $");
 #ifdef HAVE_DLFCN_H
 #include <dlfcn.h>
 #endif
@@ -117,11 +117,23 @@ loadlib(krb5_context context,
 }
 #endif /* HAVE_DLOPEN */
 
+/**
+ * Register a plugin symbol name of specific type.
+ * @param context a Keberos context
+ * @param type type of plugin symbol
+ * @param name name of plugin symbol
+ * @param symbol a pointer to the named symbol
+ * @return In case of error a non zero error com_err error is returned
+ * and the Kerberos error string is set.
+ *
+ * @ingroup krb5_support
+ */
+
 krb5_error_code
-_krb5_plugin_register(krb5_context context,
-		      enum krb5_plugin_type type,
-		      const char *name, 
-		      void *symbol)
+krb5_plugin_register(krb5_context context,
+		     enum krb5_plugin_type type,
+		     const char *name, 
+		     void *symbol)
 {
     struct plugin *e;
 
@@ -250,4 +262,3 @@ _krb5_plugin_free(struct krb5_plugin *list)
 	list = next;
     }
 }
-
