@@ -38,7 +38,7 @@
 #include <roken.h>
 #include <getarg.h>
 
-RCSID("$Id: test_ntlm.c 21602 2007-07-17 06:18:11Z lha $");
+RCSID("$Id: test_ntlm.c 22377 2007-12-28 18:38:53Z lha $");
 
 #include <krb5.h>
 #include <heimntlm.h>
@@ -59,7 +59,7 @@ test_parse(void)
     
     memset(&type1, 0, sizeof(type1));
 
-    type1.flags = NTLM_NEG_UNICODE|NTLM_NEG_NTLM;
+    type1.flags = NTLM_NEG_UNICODE|NTLM_NEG_TARGET|NTLM_NEG_NTLM;
     type1.domain = rk_UNCONST(domain);
     type1.hostname = NULL;
     type1.os[0] = 0;
@@ -84,7 +84,7 @@ test_parse(void)
 
     memset(&type2, 0, sizeof(type2));
 
-    flags = NTLM_NEG_UNICODE | NTLM_NEG_NTLM | NTLM_NEG_TARGET_DOMAIN;
+    flags = NTLM_NEG_UNICODE | NTLM_NEG_NTLM | NTLM_TARGET_DOMAIN;
     type2.flags = flags;
 
     memset(type2.challange, 0x7f, sizeof(type2.challange));
@@ -139,6 +139,16 @@ test_parse(void)
     if (ret)
 	errx(1, "heim_ntlm_decode_type3");
 
+    if (strcmp("workstation", type3.ws) != 0)
+	errx(1, "type3 ws wrong");
+
+    if (strcmp(target, type3.targetname) != 0)
+	errx(1, "type3 targetname wrong");
+
+    if (strcmp(user, type3.username) != 0)
+	errx(1, "type3 username wrong");
+
+
     heim_ntlm_free_type3(&type3);
 
     /*
@@ -147,7 +157,7 @@ test_parse(void)
 
     memset(&type2, 0, sizeof(type2));
 
-    flags = NTLM_NEG_UNICODE | NTLM_NEG_NTLM | NTLM_NEG_TARGET_DOMAIN;
+    flags = NTLM_NEG_UNICODE | NTLM_NEG_NTLM | NTLM_TARGET_DOMAIN;
     type2.flags = flags;
 
     memset(type2.challange, 0x7f, sizeof(type2.challange));
