@@ -33,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: ndbm_wrap.c,v 1.4 2005/04/12 11:28:57 lha Exp $");
+RCSID("$Id: ndbm_wrap.c 21634 2007-07-17 11:30:36Z lha $");
 #endif
 
 #include "ndbm_wrap.h"
@@ -96,8 +96,10 @@ dbm_fetch (DBM *db, datum dkey)
 #ifdef HAVE_DB3
 	       NULL, 
 #endif
-	       &key, &value, 0) != 0) 
+	       &key, &value, 0) != 0) {
 	dvalue.dptr = NULL;
+	dvalue.dsize = 0;
+    }
     else
 	DBT2DATUM(&value, &dvalue);
 
@@ -112,9 +114,10 @@ dbm_get (DB *db, int flags)
 #ifdef HAVE_DB3
     if(cursor == NULL) 
 	db->cursor(db, NULL, &cursor, 0);
-    if(cursor->c_get(cursor, &key, &value, flags) != 0) 
+    if(cursor->c_get(cursor, &key, &value, flags) != 0) {
 	datum.dptr = NULL;
-    else 
+	datum.dsize = 0;
+    } else 
 	DBT2DATUM(&value, &datum);
 #else
     db->seq(db, &key, &value, flags);
