@@ -1,6 +1,6 @@
 #!/bin/sh
 # Fetches, builds and store the result of a heimdal build
-# Version: $Id: heimdal-build.sh 20357 2007-04-16 11:14:24Z lha $
+# Version: $Id: heimdal-build.sh 21653 2007-07-18 20:15:59Z lha $
 
 fetchmethod=wget	   #options are: wget, curl, ftp, afs
 resultdir=
@@ -205,8 +205,8 @@ if [ X"$unpack" = Xyes ]; then
 fi
 
 if [ X"$autotools" = Xyes ]; then
-	echo "Autotooling (via fix-export)"
-	env DATEDVERSION="svn${branch}-${date}" ${hversion}/fix-export ${hversion}
+	echo "Autotooling"
+	(cd ${hversion} && sh ./autogen.sh) || exit 1
 fi
 
 if [ X"$ccachedir" != X ]; then
